#include "audiotag.h"
#include <stdlib.h>
#include <pthread.h>
#include <unistd.h>
#include <math.h>
#include <sys/select.h>

typedef struct {
  float *min;
  float *max;
} compute_channel;

typedef struct compute_data {
  gui *g;
  input *r;
  player *p;
  volatile int * volatile turn;
  int pipe_slow_read;
  int pipe_slow_write;
  int pipe_fast_read;
  int pipe_fast_write;
  int timer_write;
  int timer_read;
  chunk *minmax_slow;
  chunk *minmax_fast;
  compute_channel *c;
  float *buf_slow;
  float *buf_fast;
  int64_t (*get_start)(struct compute_data *);
  int64_t (*get_inc)(struct compute_data *);
} compute_data;

static int64_t get_start_hi(compute_data *d)
{
  return 0;
}

static int64_t get_start_lo(compute_data *d)
{
  return GET(d->g->start_sample);
}

static int64_t get_inc_hi(compute_data *d)
{
  return ceil((double)d->r->frames / d->g->width);
}

static int64_t get_inc_lo(compute_data *d)
{
  return GET(d->g->samples_per_pixel);
}

#include <stdio.h>
static void *slow_compute(void *_d)
{
  compute_data *d = _d;
  char b[4096];
  int64_t start, inc, rinc;
  int turn;
  int n;
  int mode;
  int i;
  int chan;
  void *in = open_audio_file(d->p->audio_file, NULL);

next:
  n = read(d->pipe_slow_read, b, 4096); if (n <= 0) abort();
  mode = b[n-1]; if (!(mode == 1 || mode == 2)) goto next;

  /* start timer if mode is do_fast (mode==2) */
  b[0] = 1; if (mode == 2) if (write(d->timer_write, b, 1)!=1) abort();

  start = d->get_start(d); //0;
  inc = d->get_inc(d);     //ceil((double)d->r->frames / d->g->width);
  turn = GET(*d->turn);
  for (i = 0; i < d->g->width; i++) {
    if (start >= 0 && start < d->r->frames) {
      rinc = inc; if (start + rinc > d->r->frames) rinc = d->r->frames - start;
      minmax(d->r, start, rinc, d->minmax_slow, in, d->buf_slow);
      if (turn != GET(*d->turn)) goto done;
      for (chan = 0; chan < d->r->channels; chan++) {
        d->c[chan].min[i] = d->minmax_slow[chan].min;
        d->c[chan].max[i] = d->minmax_slow[chan].max;
      }
    }
    start += inc;
  }
  com_redraw_gui_now();

done:
  /* stop timer if mode is do_fast (mode==2) */
  b[0] = 0; if (mode == 2) if (write(d->timer_write, b, 1)!=1) abort();
  goto next;

  return 0;
}

static void *fast_compute(void *_d)
{
  compute_data *d = _d;
  char b[4096];
  int64_t start, inc, rinc;
  int turn;
  int n;
  int mode;
  int i;
  int chan;

next:
  n = read(d->pipe_fast_read, b, 4096); if (n <= 0) abort();
  mode = b[n-1]; if (!(mode == 1 || mode == 2)) goto next;

  start = d->get_start(d);
  inc = d->get_inc(d);
  turn = GET(*d->turn);
  for (i = 0; i < d->g->width; i++) {
    if (start >= 0 && start < d->r->frames) {
      rinc = inc; if (start + rinc > d->r->frames) rinc = d->r->frames - start;
      minmax_fast(d->r, start, d->minmax_fast);
      if (turn != GET(*d->turn)) goto next;
      for (chan = 0; chan < d->r->channels; chan++) {
        d->c[chan].min[i] = d->minmax_fast[chan].min;
        d->c[chan].max[i] = d->minmax_fast[chan].max;
      }
    }
    start += inc;
  }
  com_redraw_gui_now();

  /* start slow compute (mode 1) if mode==2 */
  b[0] = 1; if (mode == 2) if (write(d->pipe_slow_write, b, 1)!=1) abort();
  goto next;

  return 0;
}

static void *timer(void *_d)
{
  compute_data *d = _d;
  char b[4096];
  fd_set rd;
  int n;

next:
  n = read(d->timer_read, b, 4096); if (n <= 0) abort();
  if (b[n-1] != 1) goto next;

  usleep(timer_length*1000);
  FD_ZERO(&rd);
  FD_SET(d->timer_read, &rd);
  if ((n=select(d->timer_read+1, &rd, NULL, NULL, NULL)) < 0) abort();
  /* if nothing to read, timeout */
  if (!n) goto timeout;
  n = read(d->timer_read, b, 4096); if (n <= 0) abort();
  /* if last is not 0, timeout */
  if (b[n-1] == 0) goto next;

timeout:
  /* timeout, start fast compute */
  ADD(*d->turn, 1);
  b[0] = 2; if (write(d->pipe_fast_write, b, 1) != 1) abort();
  goto next;
  return 0;
}

void run_compute(gui *g, input *r, player *p)
{
  pthread_attr_t attr;
  pthread_t      tt;
  compute_data *d;
  int i;
  int pp[2];

  /* hi stuff */
  d = malloc(sizeof(compute_data)); if (d==NULL) abort();
  d->g = g;
  d->r = r;
  d->p = p;
  d->turn = &hi_turn;
  d->pipe_slow_read = slow_compute_hi_read;
  d->pipe_slow_write = slow_compute_hi_write;
  d->pipe_fast_read = fast_compute_hi_read;
  d->pipe_fast_write = fast_compute_hi_write;
  d->minmax_slow = malloc(sizeof(chunk) * r->channels);
  d->minmax_fast = malloc(sizeof(chunk) * r->channels);
  if (d->minmax_slow==NULL || d->minmax_fast==NULL) abort();
  d->c = malloc(sizeof(compute_channel) * r->channels); if(d->c==NULL)abort();
  for (i = 0; i < r->channels; i++)
    d->c[i].min = g->c[i].all_min,
    d->c[i].max = g->c[i].all_max;
  if (pipe(pp)) abort();
  d->timer_read = pp[0];
  d->timer_write= pp[1];
  d->buf_slow = malloc(sizeof(float) * 4096);
  d->buf_fast = malloc(sizeof(float) * 4096);
  if (d->buf_slow==NULL || d->buf_fast==NULL) abort();
  d->get_start = get_start_hi;
  d->get_inc = get_inc_hi;

  if (pthread_attr_init(&attr)) abort();
  if (pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED)) abort();
  if (pthread_create(&tt, &attr, slow_compute, d)) abort();
  if (pthread_attr_destroy(&attr)) abort();

  if (pthread_attr_init(&attr)) abort();
  if (pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED)) abort();
  if (pthread_create(&tt, &attr, fast_compute, d)) abort();
  if (pthread_attr_destroy(&attr)) abort();

  if (pthread_attr_init(&attr)) abort();
  if (pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED)) abort();
  if (pthread_create(&tt, &attr, timer, d)) abort();
  if (pthread_attr_destroy(&attr)) abort();

  /* lo stuff */
  d = malloc(sizeof(compute_data)); if (d==NULL) abort();
  d->g = g;
  d->r = r;
  d->p = p;
  d->turn = &lo_turn;
  d->pipe_slow_read = slow_compute_lo_read;
  d->pipe_slow_write = slow_compute_lo_write;
  d->pipe_fast_read = fast_compute_lo_read;
  d->pipe_fast_write = fast_compute_lo_write;
  d->minmax_slow = malloc(sizeof(chunk) * r->channels);
  d->minmax_fast = malloc(sizeof(chunk) * r->channels);
  if (d->minmax_slow==NULL || d->minmax_fast==NULL) abort();
  d->c = malloc(sizeof(compute_channel) * r->channels); if(d->c==NULL)abort();
  for (i = 0; i < r->channels; i++)
    d->c[i].min = g->c[i].window_min,
    d->c[i].max = g->c[i].window_max;
  if (pipe(pp)) abort();
  d->timer_read = compute_lo_timer_read;
  d->timer_write= compute_lo_timer_write;
  d->buf_slow = malloc(sizeof(float) * 4096);
  d->buf_fast = malloc(sizeof(float) * 4096);
  if (d->buf_slow==NULL || d->buf_fast==NULL) abort();
  d->get_start = get_start_lo;
  d->get_inc = get_inc_lo;

  if (pthread_attr_init(&attr)) abort();
  if (pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED)) abort();
  if (pthread_create(&tt, &attr, slow_compute, d)) abort();
  if (pthread_attr_destroy(&attr)) abort();

  if (pthread_attr_init(&attr)) abort();
  if (pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED)) abort();
  if (pthread_create(&tt, &attr, fast_compute, d)) abort();
  if (pthread_attr_destroy(&attr)) abort();

  if (pthread_attr_init(&attr)) abort();
  if (pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED)) abort();
  if (pthread_create(&tt, &attr, timer, d)) abort();
  if (pthread_attr_destroy(&attr)) abort();
}
