#include <stdio.h>
#include <stdlib.h>
#include <sndfile.h>
#include "audiotag.h"

void minmax_fast(input *r, uint64_t start, chunk *minmax)
{
  int i;
  for (i = 0; i < r->channels; i++)
    minmax[i] = r->chunks[start/4096 * r->channels + i];
}

static void load(input *r, uint64_t start, uint64_t count,
                 SNDFILE *in, float *buf)
{
  sf_count_t c;

  c = sf_seek(in, start, SEEK_SET);
  if (c != start) { fprintf(stderr, "error sf_seek\n"); exit(1); }

  c = sf_read_float(in, buf, count * r->channels);
  if (c != count * r->channels)
    { fprintf(stderr, "error sf_read_float start %ld count %ld ret %ld\n", start, count, c); exit(1); }

}

void minmax(input *r, uint64_t start, uint64_t len, chunk *minmax,
            void *_in, float *buf)
{
  uint64_t count;
  unsigned int i;
  chunk *cur;
  int chan;

  for (chan = 0; chan < r->channels; chan++) {
    minmax[chan].min = 1e10;
    minmax[chan].max = -1e10;
  }

  if (start & 4095) {
    count = 4096 - (start & 4095);
    if (count > len) count = len;
    load(r, start, count, _in, buf);
    for (chan = 0; chan < r->channels; chan++)
    for (i = 0; i < count; i++) {
      if (buf[chan + i * r->channels] < minmax[chan].min)
        minmax[chan].min = buf[chan + i * r->channels];
      if (buf[chan + i * r->channels] > minmax[chan].max)
        minmax[chan].max = buf[chan + i * r->channels];
    }
    len -= count;
    start += count;
  }
  if (!len) return;

  cur = &r->chunks[start/4096 * r->channels];
  while (len >= 4096) {
    for (chan = 0; chan < r->channels; chan++, cur++) {
      if (cur[0].min < minmax[chan].min)
        minmax[chan].min = cur[0].min;
      if (cur[0].max > minmax[chan].max)
        minmax[chan].max = cur[0].max;
    }
    len -= 4096;
    start += 4096;
  }

  if (len) {
    load(r, start, len, _in, buf);
    for (chan = 0; chan < r->channels; chan++)
    for (i = 0; i < len; i++) {
      if (buf[chan + i * r->channels] < minmax[chan].min)
        minmax[chan].min = buf[chan + i * r->channels];
      if (buf[chan + i * r->channels] > minmax[chan].max)
        minmax[chan].max = buf[chan + i * r->channels];
    }
  }
}
