#include <stdio.h>
#include <sndfile.h>
#include <stdlib.h>
#include "audiotag.h"

int main(int n, char **v)
{
  SNDFILE *in;
  FILE *out;
  SF_INFO in_info;
  float *buf;
  chunk cur;
  int i, j;

  if (n != 3) { fprintf(stderr, "gimme <in file> <out file>\n"); return 1; }

  in_info.format = 0;
  in = sf_open(v[1], SFM_READ, &in_info);
  if (in == NULL) { fprintf(stderr, "error opening %s\n", v[1]); return 1; }

  buf = malloc(sizeof(float) * 4096 * in_info.channels);
  if (buf == NULL) { fprintf(stderr, "out of memory\n"); return 1; }

  out = fopen(v[2], "w"); if (out == NULL) { perror(v[2]); return 1; }

  int nb_chunks = 0;
  while (in_info.frames) {
    sf_count_t count;
    sf_count_t to_read;
    to_read = in_info.frames > 4096 ? 4096 : in_info.frames;
    count = sf_read_float(in, buf, to_read * in_info.channels);
    if (count != to_read * in_info.channels) {
      fprintf(stderr, "error reading %s (%s)\n", v[1], sf_strerror(in));
      return 1;
    }
    in_info.frames -= to_read;
    /* we suppose interleaved data */
    for (i = 0; i < in_info.channels; i++) {
      cur.min = buf[i];
      cur.max = buf[i];
      for (j = 1; j < to_read; j++) {
        float v = buf[i + j * in_info.channels];
        if (v < cur.min) cur.min = v;
        if (v > cur.max) cur.max = v;
      }
      if (fwrite(&cur, sizeof(chunk), 1, out) != 1) {
        fprintf(stderr, "error writing %s\n", v[2]);
        return 1;
      }
      nb_chunks++;
    }
  }
  printf("%d chunks\n", nb_chunks);

  if (fclose(out)) { perror(v[2]); return 1; }

  return 0;
}
